package com.qmth.teachcloud.common.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicAttachment;
import com.qmth.teachcloud.common.enums.UploadFileEnum;
import com.qmth.teachcloud.common.mapper.BasicAttachmentMapper;
import com.qmth.teachcloud.common.service.BasicAttachmentService;
import com.qmth.teachcloud.common.util.OssUtil;
import com.qmth.teachcloud.common.util.ResultUtil;
import org.apache.commons.codec.digest.DigestUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.FileCopyUtils;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.List;
import java.util.Objects;
import java.util.StringJoiner;

/**
 * <p>
 * 所有附件记录表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class BasicAttachmentServiceImpl extends ServiceImpl<BasicAttachmentMapper, BasicAttachment> implements BasicAttachmentService {
    private final static Logger log = LoggerFactory.getLogger(BasicAttachmentServiceImpl.class);

    @Resource
    OssUtil ossUtil;

    /**
     * 保存html附件
     *
     * @param fileName
     * @param htmlContent
     * @return
     * @throws IOException
     */
    @Override
    public File saveAttachmentHtml(String rootPath, String fileName, String htmlContent) {
        File file = null;
        try {
            byte[] bytes = htmlContent.getBytes();
            //上传至服务器
            File mkdir = new File(rootPath);
            if (!mkdir.exists()) {
                mkdir.mkdirs();
            }
            file = new File(rootPath, fileName);
            FileCopyUtils.copy(bytes, file);
            //计算html文件md5
        } catch (Exception e) {
            e.printStackTrace();
        }
        return file;
    }

    /**
     * 保存附件
     *
     * @param dirName
     * @param userId
     * @return
     * @throws IOException
     */
    @Override
    @Transactional
    public BasicAttachment saveAttachmentPdf(String dirName, Long userId) throws IOException {
        BasicAttachment basicAttachment = null;
        try {
            StringJoiner stringJoiner = new StringJoiner("").add(SystemConstant.PDF_TEMP_FILES_DIR).add(File.separator);
            File localPdfFile = new File(stringJoiner.toString() + File.separator + dirName);
            if (!localPdfFile.getParentFile().exists()) {
                localPdfFile.getParentFile().mkdirs();
                localPdfFile.createNewFile();
            }
            JSONObject jsonPdf = new JSONObject();
            jsonPdf.put(SystemConstant.TYPE, SystemConstant.OSS);
            jsonPdf.put(SystemConstant.PATH, dirName);
            jsonPdf.put(SystemConstant.UPLOAD_TYPE, UploadFileEnum.PDF);
            basicAttachment = new BasicAttachment(jsonPdf.toJSONString(), localPdfFile.getName(), SystemConstant.PDF_PREFIX, new BigDecimal(localPdfFile.length()), DigestUtils.md5Hex(new FileInputStream(localPdfFile)), userId);
            save(basicAttachment);
        } catch (Exception e) {
            log.error("请求出错", e);
            deleteAttachment(basicAttachment);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return basicAttachment;
    }

    /**
     * 删除附件
     *
     * @param basicAttachment
     */
    @Override
    public void deleteAttachment(BasicAttachment basicAttachment) {
        deleteAttachmentCommon(basicAttachment);
    }

    /**
     * 批量删除附件
     *
     * @param basicAttachmentList
     */
    @Override
    public void batchDeleteAttachment(List<BasicAttachment> basicAttachmentList) {
        if (Objects.nonNull(basicAttachmentList) && basicAttachmentList.size() > 0) {
            for (BasicAttachment basicAttachment : basicAttachmentList) {
                deleteAttachmentCommon(basicAttachment);
            }
        }
    }

    /**
     * 删除附件公用
     *
     * @param basicAttachment
     */
    public void deleteAttachmentCommon(BasicAttachment basicAttachment) {
        if (Objects.nonNull(basicAttachment) && Objects.nonNull(basicAttachment.getPath())) {
            JSONObject jsonObject = JSONObject.parseObject(basicAttachment.getPath());
            String type = String.valueOf(jsonObject.get(SystemConstant.TYPE));
            if (Objects.nonNull(type) && Objects.equals(type, SystemConstant.OSS)) {//删除阿里云附件
                ossUtil.ossDelete(jsonObject.get(SystemConstant.PATH).toString());
                if (Objects.nonNull(jsonObject.get(SystemConstant.PDF_PATH))) {
                    ossUtil.ossDelete(jsonObject.get(SystemConstant.PDF_PATH).toString());
                }
            } else {//删除服务器附件
                File file = new File(jsonObject.get(SystemConstant.PATH).toString());
                file.delete();
                if (Objects.nonNull(jsonObject.get(SystemConstant.PDF_PATH))) {
                    file = new File(jsonObject.get(SystemConstant.PDF_PATH).toString());
                    file.delete();
                }
            }
        }
    }
}
